<?php namespace EwTools\Admin\SettingsPage;

use EwTools\Handler\JsonImporter;
use EwTools\Interfaces\SettingsPageInterface;
use EwTools\Models\Section;
use EwTools\Support\Str;
use EwTools\Twig\View;

/**
 * SectionsPage
 *
 * @link    http://infomaker.se
 * @package EwTools\Admin\SettingsPage
 * @since   EwTools\Admin\SettingsPage\SectionsPage 0.1
 */
class SectionsPage implements SettingsPageInterface {
    
    protected static $slug = 'sections';
    
    protected static $title = 'Sections';
    
    protected $text_domain = 'ewtools_textdomain';
    
    /**
     * Contains in instance of the JsonImporter class
     *
     * @var JsonImporter
     */
    protected $json_importer;
    
    /**
     * Retrieve any messages that may be displayed by the settings-page
     *
     * @since 0.1
     * @return string
     */
    public function getMessage() {
        return $this->isUpdated() ? 'Updated! Changes have been saved.' : $this->json_importer->isImported() ? 'Import done! Changes have not yet been saved.' : '';
    }
    
    /**
     * Retrieve the slug to be used to identify and activate tab
     *
     * @since 0.1
     * @return string
     */
    public function getTabSlug() {
        return Str::slug( static::$slug );
    }
    
    /**
     * The title/label displayed on the tab
     *
     * @since 0.1
     * @return string
     */
    public function getTabTitle() {
        return __( static::$title, $this->text_domain );
    }
    
    /**
     * Retrieve and handle the data sent from the admin page
     *
     * @since 0.1
     * @return Collection
     */
    protected function getSubmittedData() {
        $submitted_data = array_get( $_POST, 'field', [] );
        
        $valid_data = [];
        foreach ( $submitted_data as $field => $values ) {
            foreach ( array_filter( $values ) as $index => $value ) {
                if( ! isset( $valid_data[ $index ] ) ) {
                    $valid_data[ $index ] = [];
                }
                
                $valid_data[ $index ][ $field ] = $value;
            }
        }
        
        return Section::makeCollection( $valid_data, 'name' );
    }
    
    /**
     * Determine if form has been submitted
     *
     * @since 0.1
     * @return bool
     */
    protected function isUpdated() {
        return isset( $_POST[ 'everyware-sections-submit' ] );
    }
    
    /**
     * Fires on Page-load if active
     *
     * @since 0.1
     * @return void
     */
    public function onPageLoad() {
        $this->json_importer = new JsonImporter();
        if( $this->isUpdated() ) {
            Section::collection()->setCollection( $this->getSubmittedData() )->save();
        }
    }
    
    /**
     * Fires on Page-load if active
     *
     * @since 0.1
     * @return string
     */
    public function pageContent() {
        $imported_sections = Section::makeCollection( $this->json_importer->getJson(), 'name' );
        
        $sections = $this->json_importer->isImported() ? Section::collection()->merge( $imported_sections ) : Section::all();
        
        return View::generate( '@tools/admin/settings-page/sections', [
            'form_action' => $_SERVER[ 'REQUEST_URI' ],
            'text_domain' => $this->text_domain,
            'template'    => [
                'Name'       => '',
                'Class Name' => '',
                'Includes'   => ''
            ],
            'sections'    => $sections->map( function ( $section ) {
                return [
                    'Name'       => $section->name,
                    'Class Name' => $section->class_name,
                    'Includes'   => $section->includes
                ];
            } )->toArray(),
            'json_import' => $this->json_importer->generate( $imported_sections->toJson(), Section::all()->toJson() )
        ] );
    }
}