<?php namespace EwTools\Admin\SettingsPage;

use EwTools\Handler\JsonImporter;
use EwTools\Interfaces\SettingsPageInterface;
use EwTools\Models\SettingsParameter;
use EwTools\Support\Arr;
use EwTools\Support\Collection;
use EwTools\Support\Str;
use EwTools\Twig\View;

/**
 * SettingsParametersPage
 *
 * @link    http://infomaker.se
 * @package EwTools\Admin\SettingsPage
 * @since   EwTools\Admin\SettingsPage\SettingsParametersPage 0.1
 */
class SettingsParametersPage implements SettingsPageInterface {
    
    /**
     * @var string
     */
    protected static $slug = 'settings-parameters';
    
    /**
     * @var string
     */
    protected static $title = 'Settings Parameters';
    
    /**
     * @var string
     */
    protected $text_domain = 'ewtools_textdomain';
    
    /**
     * Contains in instance of the JsonImporter class
     *
     * @var JsonImporter
     */
    protected $json_importer;
    
    public function __construct() {
        if( is_admin() ) {
            add_action( 'wp_ajax_everyware_delete_setting_parameter', [ $this, 'ajax_delete_setting_parameter' ] );
        }
    }
    
    /**
     * Retrieve the slug to be used to identify and activate tab
     *
     * @since 0.1
     * @return string
     */
    public function getTabSlug() {
        return Str::slug( static::$slug );
    }
    
    /**
     * The title/label displayed on the tab
     *
     * @since 0.1
     * @return string
     */
    public function getTabTitle() {
        return __( static::$title, $this->text_domain );
    }
    
    /**
     * Retrieve and handle the parameters sent from the admin page
     *
     * @since 0.1
     * @return Collection
     */
    protected function getSubmittedParameters() {
        $submitted_parameters = Arr::get( $_POST, 'field', [] );
        
        $valid_parameters = [];
        foreach ( $submitted_parameters as $field => $values ) {
            foreach ( array_filter( $values ) as $index => $value ) {
                if( ! isset( $valid_parameters[ $index ] ) ) {
                    $valid_parameters[ $index ] = [];
                }
                
                $valid_parameters[ $index ][ $field ] = $value;
            }
        }
        
        return SettingsParameter::makeCollection( $valid_parameters, 'key' );
    }
    
    /**
     * Determine if form has been submitted
     *
     * @since 0.1
     * @return bool
     */
    protected function isUpdated() {
        return isset( $_POST[ 'everyware-settings-parameters-submit' ] );
    }
    
    /**
     * Fires on Page-load if active
     *
     * @since 0.1
     * @return void
     */
    public function onPageLoad() {
        $this->json_importer = new JsonImporter();
        
        if( $this->isUpdated() ) {
            SettingsParameter::collection()->setCollection( $this->getSubmittedParameters() )->save();
        }
    }
    
    /**
     * Retrieve any messages that may be displayed by the settings-page
     *
     * @since 0.1
     * @return string
     */
    public function getMessage() {
        return $this->isUpdated() ? 'Updated! Changes have been saved.' : $this->json_importer->isImported() ? 'Import done! Changes have not yet been saved.' : '';
    }
    
    /**
     * Fires on Page-load if active
     *
     * @since 0.1
     * @return string
     */
    public function pageContent() {
        $imported_parameters = SettingsParameter::makeCollection( $this->json_importer->getJson(), 'key' );
        
        $parameters = $this->json_importer->isImported() ? SettingsParameter::collection()->merge( $imported_parameters ) : SettingsParameter::all();
        
        return View::generate( '@tools/admin/settings-page/settings-parameters', [
            'form_action' => $_SERVER[ 'REQUEST_URI' ],
            'text_domain' => $this->text_domain,
            'template'    => [
                'Key'   => '',
                'Value' => '',
            ],
            'parameters'  => $parameters->map( function ( $parameter ) {
                return [
                    'Key'   => $parameter->key,
                    'Value' => $parameter->value,
                ];
            } )->toArray(),
            'json_import' => $this->json_importer->generate( $imported_parameters->toJson(), SettingsParameter::all()->toJson() )
        ] );
    }
}