<?php
/**
 * Handler: Handle the menus of the project.
 */
namespace EwTools\Handler;

use EwTools\Interfaces\Support\Selectable;
use EwTools\Models\Menu;
use EwTools\Support\Collection;
use EwTools\Support\Str;
use WP_Term;

/**
 * Menus
 *
 * @link    http://infomaker.se
 * @package WWW\Classes\EwTools\Handler
 * @since   WWW\Classes\EwTools\Handler\Menus 0.1
 */
class Menus implements Selectable {
    
    /**
     * Contains the defaults for fetching menus via WP
     *
     * @var array
     */
    protected static $defaults = [
        'taxonomy'               => 'nav_menu',
        'orderby'                => 'name',
        'order'                  => 'ASC',
        'hide_empty'             => false,
        'include'                => [],
        'exclude'                => [],
        'exclude_tree'           => [],
        'number'                 => '',
        'offset'                 => '',
        'fields'                 => 'all',
        'name'                   => '',
        'slug'                   => '',
        'hierarchical'           => true,
        'search'                 => '',
        'name__like'             => '',
        'description__like'      => '',
        'pad_counts'             => false,
        'get'                    => '',
        'child_of'               => 0,
        'parent'                 => '',
        'childless'              => false,
        'cache_domain'           => 'core',
        'update_term_meta_cache' => true,
        'meta_query'             => ''
    ];
    
    /**
     * Contains the registered menu locations
     *
     * @var array
     */
    public static $locations = [];
    
    /**
     * Retrieves all menus
     *
     * @param array $args Optional. Array of arguments passed on to {@see get_terms()}.
     *                    Default empty array.
     *
     * @since 0.1
     * @return array|Collection
     */
    public static function all( $args = [] ) {
        $menus = wp_get_nav_menus( array_replace_recursive( static::$defaults, $args ) );
        
        if( is_array( $menus ) ) {
            return new Collection( array_map( function ( WP_Term $menu ) {
                return new Menu($menu);
            }, $menus ) );
        }
        
        return [];
    }
    
    /**
     * Function to retrieve all menus as select-box data
     *
     * @since 0.1
     * @return array
     */
    public static function getSelectData() {
        return static::toSelect();
    }
    
    /**
     * Use filter to fetch a menu
     *
     * @param array $filter 'name', 'id', 'slug' or 'location'
     *
     * @since 0.1
     * @return static|null
     */
    public static function where( $filter = [] ) {
        if( empty( $filter ) ) {
            return null;
        }
        
        $valid_filters = [ 'name', 'id', 'slug', 'location' ];
        
        foreach ( array_keys( $filter ) as $filter_key ) {
            if( in_array( $filter_key, $valid_filters ) ) {
                return Str::is( 'location', $filter_key ) ? static::getFromLocation( $filter[ $filter_key ] ) : static::getMenu( $filter[ $filter_key ] );
            }
        }
        
        return null;
    }
    
    /**
     * Function to fetch a menu that may be attached to a menu-location
     *
     * @param $location
     *
     * @since 0.1
     * @return bool|Menu
     */
    public static function getFromLocation( $location ) {
        if ( has_nav_menu($location) ) {
            $theme_locations = static::getLocations();
            
            if( isset( $theme_locations[ $location ] ) ) {
                $menu_obj = get_term( $theme_locations[ $location ], 'nav_menu' );
                
                return ! is_wp_error( $menu_obj ) ? new Menu( $menu_obj ) : false;
            }
        }
        
        return false;
    }
    
    /**
     * Retrieve a menu
     *
     * @param $menu
     *
     * @since 0.1
     * @return bool|Menu
     */
    public static function getMenu( $menu ) {
        $menu_obj = wp_get_nav_menu_object( $menu );
        
        return ! is_wp_error( $menu_obj ) ? new Menu( $menu_obj ) : false;
    }
    
    /**
     * Add multiple menu locations at once
     *
     * @param array $locations
     *
     * @since 0.1
     * @return void
     */
    public static function addLocations( array $locations ) {
        foreach ( $locations as $location ) {
            if ( isset($location['id'], $location['description']) ) {
                static::addLocation( $location['id'], $location['description'] );
            }
        }
    }
    
    /**
     * Add a menu location to be registered in Wordpress
     *
     * @param $id
     * @param $description
     *
     * @since 0.1
     * @return void
     */
    public static function addLocation( $id, $description ) {
        static::$locations[$id] = $description;
    }
    
    /**
     * Retrieve registered menu locations
     *
     * @uses {get_nav_menu_locations()}
     *
     * @since 0.1
     * @return array
     */
    public static function getLocations( ) {
        return get_nav_menu_locations();
    }
    
    /**
     * Initiate menus
     *
     * @param null $locations
     *
     * @since 0.1
     * @return void
     */
    public static function init( $locations = null) {
        
        // If there are locations to be added before registration
        if ( is_array($locations) ) {
            static::addLocations($locations);
        }
    
        /**
         * Register the menu locations in Wordpress
         */
        add_action( 'init', function() {
            foreach ( static::$locations as $id => $description ) {
                register_nav_menu( $id, $description );
            }
        } );
    }
    
    /**
     * Return data in the form of an array fit for a select-box
     * A preselected value may be sent in.
     *
     * @param mixed $selected
     *
     * @since 0.1
     * @return array
     */
    public static function toSelect( $selected = null ) {
        return static::all()->map(function ( $menu ) use ($selected) {
            return [
                'value' => $menu->term_id,
                'text'  => $menu->name,
                'selected' => $menu->term_id === $selected
            ];
        });
    }
}