<?php
/**
 * Support: for advanced array handling
 */
namespace EwTools\Support;

use EwTools\Support\Traits\Macroable;

/**
 * Arr
 *
 * @link    http://infomaker.se
 * @package EwTools\Support
 * @since   EwTools\Support\Arr 0.1
 */
class Arr {
	
	use Macroable;
	
	/**
	 * Add an element to an array using "dot" notation if it doesn't exist.
	 *
	 * @param array  $array
	 * @param string $key
	 * @param mixed  $value
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function add( $array, $key, $value ) {
		if ( is_null( static::get( $array, $key ) ) ) {
			static::set( $array, $key, $value );
		}
		
		return $array;
	}
	
	/**
	 * Collapse an array of arrays into a single array.
	 *
	 * @param array|\ArrayAccess $array
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function collapse( $array ) {
		$results = [];
		
		foreach ( $array as $values ) {
			if ( $values instanceof Collection ) {
				$values = $values->all();
			}
			
			if ( ! is_array( $values ) ) {
				continue;
			}
			
			$results = array_merge( $results, $values );
		}
		
		return $results;
	}
	
	/**
	 * Divide an array into two arrays. One with keys and the other with values.
	 *
	 * @param array $array
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function divide( $array ) {
		return [ array_keys( $array ), array_values( $array ) ];
	}
	
	/**
	 * Flatten a multi-dimensional associative array with dots.
	 *
	 * @param array  $array
	 * @param string $prepend
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function dot( $array, $prepend = '' ) {
		$results = [];
		
		foreach ( $array as $key => $value ) {
			if ( is_array( $value ) ) {
				$results = array_merge( $results, static::dot( $value, $prepend . $key . '.' ) );
			} else {
				$results[ $prepend . $key ] = $value;
			}
		}
		
		return $results;
	}
	
	/**
	 * Get all of the given array except for a specified array of items.
	 *
	 * @param array        $array
	 * @param array|string $keys
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function except( $array, $keys ) {
		static::forget( $array, $keys );
		
		return $array;
	}
	
	/**
	 * Return the first element in an array passing a given truth test.
	 *
	 * @param array    $array
	 * @param callable $callback
	 * @param mixed    $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public static function first( $array, callable $callback, $default = null ) {
		foreach ( $array as $key => $value ) {
			if ( call_user_func( $callback, $key, $value ) ) {
				return $value;
			}
		}
		
		return ev_value( $default );
	}
	
	/**
	 * Return the last element in an array passing a given truth test.
	 *
	 * @param array    $array
	 * @param callable $callback
	 * @param mixed    $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public static function last( $array, callable $callback, $default = null ) {
		return static::first( array_reverse( $array ), $callback, $default );
	}
	
	/**
	 * Flatten a multi-dimensional array into a single level.
	 *
	 * @param array $array
	 * @param int   $depth
	 *
	 * @return array
	 */
	public static function flatten( $array, $depth = INF ) {
		return array_reduce( $array, function( $result, $item ) use ( $depth ) {
			$item = $item instanceof Collection ? $item->all() : $item;
			
			if ( is_array( $item ) ) {
				if ( $depth === 1 ) {
					return array_merge( $result, $item );
				}
				
				return array_merge( $result, static::flatten( $item, $depth - 1 ) );
			}
			
			$result[] = $item;
			
			return $result;
		}, [] );
	}
	
	/**
	 * Remove one or many array items from a given array using "dot" notation.
	 *
	 * @param array        $array
	 * @param array|string $keys
	 *
	 * @since 0.1
	 * @return void
	 */
	public static function forget( &$array, $keys ) {
		$original = &$array;
		
		$keys = (array) $keys;
		
		if ( count( $keys ) === 0 ) {
			return;
		}
		
		foreach ( $keys as $key ) {
			$parts = explode( '.', $key );
			
			// clean up before each pass
			$array = &$original;
			
			while ( count( $parts ) > 1 ) {
				$part = array_shift( $parts );
				
				if ( isset( $array[ $part ] ) && is_array( $array[ $part ] ) ) {
					$array = &$array[ $part ];
				} else {
					continue 2;
				}
			}
			
			unset( $array[ array_shift( $parts ) ] );
		}
	}
	
	/**
	 * Get an item from an array using "dot" notation.
	 *
	 * @param array  $array
	 * @param string $key
	 * @param mixed  $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public static function get( $array, $key, $default = null ) {
		if ( is_null( $key ) ) {
			return $array;
		}
		
		if ( isset( $array[ $key ] ) ) {
			return $array[ $key ];
		}
		
		foreach ( explode( '.', $key ) as $segment ) {
			if ( ! is_array( $array ) || ! array_key_exists( $segment, $array ) ) {
				return ev_value( $default );
			}
			
			$array = $array[ $segment ];
		}
		
		return $array;
	}
	
	/**
	 * Check if an item exists in an array using "dot" notation.
	 *
	 * @param array  $array
	 * @param string $key
	 *
	 * @since 0.1
	 * @return bool
	 */
	public static function has( $array, $key ) {
		if ( empty( $array ) || is_null( $key ) ) {
			return false;
		}
		
		if ( array_key_exists( $key, $array ) ) {
			return true;
		}
		
		foreach ( explode( '.', $key ) as $segment ) {
			if ( ! is_array( $array ) || ! array_key_exists( $segment, $array ) ) {
				return false;
			}
			
			$array = $array[ $segment ];
		}
		
		return true;
	}
    
    /**
     * Check whether the array has non-integer keys
     * Note: (not whether the array is sequentially-indexed or zero-indexed)
     *
     * @param array $array
     *
     * @since 0.1
     * @return bool
     */
    public static function hasStringKeys( array $array ) {
        return count(array_filter(array_keys($array), 'is_string')) > 0;
	}
	
	/**
	 * Determines if an array is associative.
	 * An array is "associative" if it doesn't have sequential numerical keys beginning with zero.
	 *
	 * @param array $array
	 *
	 * @since 0.1
	 * @return bool
	 */
	public static function isAssoc( array $array ) {
		$keys = array_keys( $array );
		
		return array_keys( $keys ) !== $keys;
	}
	
	/**
	 * Get a subset of the items from the given array.
	 *
	 * @param array        $array
	 * @param array|string $keys
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function only( $array, $keys ) {
		return array_intersect_key( $array, array_flip( (array) $keys ) );
	}
	
	/**
	 * Pluck an array of values from an array.
	 *
	 * @param array|\ArrayAccess $array
	 * @param string|array       $value
	 * @param string|array|null  $key
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function pluck( $array, $value, $key = null ) {
		$results = [];
		
		list( $value, $key ) = static::explodePluckParameters( $value, $key );
		
		foreach ( $array as $item ) {
			$itemValue = ev_data_get( $item, $value );
			
			// If the key is "null", we will just append the value to the array and keep
			// looping. Otherwise we will key the array using the value of the key we
			// received from the developer. Then we'll return the final array form.
			if ( is_null( $key ) ) {
				$results[] = $itemValue;
			} else {
				$itemKey = ev_data_get( $item, $key );
				
				$results[ $itemKey ] = $itemValue;
			}
		}
		
		return $results;
	}
	
	/**
	 * Explode the "value" and "key" arguments passed to "pluck".
	 *
	 * @param string|array      $value
	 * @param string|array|null $key
	 *
	 * @since 0.1
	 * @return array
	 */
	protected static function explodePluckParameters( $value, $key ) {
		$value = is_string( $value ) ? explode( '.', $value ) : $value;
		
		$key = is_null( $key ) || is_array( $key ) ? $key : explode( '.', $key );
		
		return [ $value, $key ];
	}
	
	/**
	 * Push an item onto the beginning of an array.
	 *
	 * @param array $array
	 * @param mixed $value
	 * @param mixed $key
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function prepend( $array, $value, $key = null ) {
		if ( is_null( $key ) ) {
			array_unshift( $array, $value );
		} else {
			$array = [ $key => $value ] + $array;
		}
		
		return $array;
	}
	
	/**
	 * Get a value from the array, and remove it.
	 *
	 * @param array  $array
	 * @param string $key
	 * @param mixed  $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public static function pull( &$array, $key, $default = null ) {
		$value = static::get( $array, $key, $default );
		
		static::forget( $array, $key );
		
		return $value;
	}
	
	/**
	 * Set an array item to a given value using "dot" notation.
	 * If no key is given to the method, the entire array will be replaced.
	 *
	 * @param array  $array
	 * @param string $key
	 * @param mixed  $value
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function set( &$array, $key, $value ) {
		if ( is_null( $key ) ) {
			return $array = $value;
		}
		
		$keys = explode( '.', $key );
		
		while ( count( $keys ) > 1 ) {
			$key = array_shift( $keys );
			
			// If the key doesn't exist at this depth, we will just create an empty array
			// to hold the next value, allowing us to create the arrays to hold final
			// values at the correct depth. Then we'll keep digging into the array.
			if ( ! isset( $array[ $key ] ) || ! is_array( $array[ $key ] ) ) {
				$array[ $key ] = [];
			}
			
			$array = &$array[ $key ];
		}
		
		$array[ array_shift( $keys ) ] = $value;
		
		return $array;
	}
	
	/**
	 * Sort the array using the given callback.
	 *
	 * @param array    $array
	 * @param callable $callback
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function sort( $array, callable $callback ) {
		return Collection::make( $array )->sortBy( $callback )->all();
	}
	
	/**
	 * Sort the array using the given callback.
	 *
	 * @param array    $array
	 * @param callable $callback
	 *
	 * @return array
	 */
	public static function rsort( $array, callable $callback ) {
		return Collection::make( $array )->sortBy( $callback, SORT_REGULAR, true )->all();
	}
	
	/**
	 * Recursively sort an array by keys and values.
	 *
	 * @param array $array
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function sortRecursive( $array ) {
		foreach ( $array as &$value ) {
			if ( is_array( $value ) ) {
				$value = static::sortRecursive( $value );
			}
		}
		
		if ( static::isAssoc( $array ) ) {
			ksort( $array );
		} else {
			sort( $array );
		}
		
		return $array;
	}
	
	/**
	 * Filter the array using the given callback.
	 *
	 * @param array    $array
	 * @param callable $callback
	 *
	 * @since 0.1
	 * @return array
	 */
	public static function where( $array, callable $callback ) {
		$filtered = [];
		
		foreach ( $array as $key => $value ) {
			if ( call_user_func( $callback, $key, $value ) ) {
				$filtered[ $key ] = $value;
			}
		}
		
		return $filtered;
	}
}
