<?php
/**
 * Twig Extension Class: for handling the environment of Twig
 */
namespace EwTools\Twig;

use Twig_Environment;
use Twig_Lexer;
use Twig_Loader_Chain;
use Twig_LoaderInterface;
use Twig_Parser;
use Twig_SimpleFunction;

/**
 * Every_Twig
 *
 * @link    http://infomaker.se
 * @package EwTools\Twig
 * @since   EwTools\Twig\Every_Twig 0.1
 */
class Every_Twig extends Twig_Environment {
	
	/**
	 * Contains an instance of the parent for calls to certain functions
	 *
	 * @var \Twig_Environment
	 */
	private static $twig;
	
	/**
	 * Contains an instance of self
	 *
	 * @var Every_Twig
	 */
	private static $instance;
	
	/**
	 * Contains a list of registered loaders
	 *
	 * @var array
	 */
	private static $loaders = [];
	
	/**
	 * Contains whether to use cache
	 *
	 * @var bool
	 */
	protected $cache = false;
	
	/**
	 * Time in seconds to cache the output
	 *
	 * @var int
	 */
	private $cache_time = 60;
	
	/**
	 * Add suffix so we don't have to delete it in deploy every time
	 *
	 * @var string
	 */
	private $cache_key = 'everytwig_compiled_cache_' . GIT_COMMIT;
	
	/**
	 * Create new instance if none exist
	 *
	 * @since 0.1
	 * @return Every_Twig
	 */
	public static function get_instance() {
		if ( null == self::$instance ) {
			self::$instance = new self;
		}
		
		return self::$instance;
	}
	
	/**
	 * Initializes the EveryTwig-wrapper and Twig environment
	 *
	 * @param Twig_LoaderInterface[] $loaders
	 * @param array                  $functions
	 * @param array                  $filters
	 * @param array                  $options
	 *
	 * @since 0.1
	 * @return Every_Twig
	 */
	public function init( array $loaders, array $functions, array $filters, array $options ) {
		$this->handle_options( $options );
		
		foreach ( $loaders as $loader ) {
			$this->add_loader( $loader );
		}
		
		if ( false === get_transient( $this->cache_key ) ) {
			set_transient( $this->cache_key, [], $this->cache_time );
		}
		
		$chain = new Twig_Loader_Chain( self::$loaders );
		$twig  = new Twig_Environment( $chain, $options );
		
		self::$twig = $twig;
		self::$twig->addFunction( $this->default_php_function() );
		
		foreach ( $functions as $function ) {
			self::$twig->addFunction( $function );
		}
		
		foreach ( $filters as $filter ) {
			self::$twig->addFilter( $filter );
		}
		
		self::$twig->setLexer( new Twig_Lexer( $this ) );
		self::$twig->setParser( new Twig_Parser( $this ) );
		
		return self::$instance;
	}
	
	/**
	 * Handles the specific cache options and trims them away from the options array.
	 *
	 * @param array $options
	 *
	 * @since 0.1
	 * @return void
	 */
	private function handle_options( &$options ) {
		if ( isset( $options['cache'] ) ) {
			$this->cache = $options['cache'];
			unset( $options['cache'] );
		}
		
		if ( isset( $options['cachetime'] ) && intval( $options['cachetime'] ) > 0 ) {
			$this->cache_time = intval( $options['cachetime'] );
			unset( $options['cachetime'] );
		}
	}
	
	/**
	 * Helper function to add a loader
	 *
	 * @param Twig_LoaderInterface $loader
	 *
	 * @since 0.1
	 * @return void
	 */
	private function add_loader( Twig_LoaderInterface $loader ) {
		array_push( self::$loaders, $loader );
	}
	
	/**
	 * Twig_SimpleFunction to be able to call default php functions from within twig templates
	 *
	 * @since 0.1
	 * @return Twig_SimpleFunction
	 */
	protected function default_php_function() {
		$phpFunction = new Twig_SimpleFunction( 'php_function', function( $name, $args = '' ) {
			if ( function_exists( $name ) ) {
				return call_user_func_array( $name, $args );
			}
			
			if ( strpos( $name, '::' ) ) {
				$static_method = explode( '::', $name );
				if ( method_exists( $static_method[0], $static_method[1] ) ) {
					return call_user_func_array( $name, $args );
				}
			}
			
		} );
		
		return $phpFunction;
	}
	
	/**
	 * Render function of EveryTwig, loads a specific template and works with cached compiled templates.
	 *
	 * @param string $template
	 * @param array  $context
	 *
	 * @since 0.1
	 * @return void
	 */
	public function render( $template, array $context = [] ) {
		echo $this->loadTemplate( $template )->render( $context );
	}
	
	/**
	 * Return the view as rendered HTML by passed template
	 *
	 * @param string $template
	 * @param array  $context
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public function view( $template, array $context = [] ) {
		return $this->loadTemplate( $template )->render( $context );
	}
	
	/**
	 * Loads a specific template and compiles it. If cache is on, save the compiled class in cac3he to be evaluated
	 * later on.
	 *
	 * @param string   $name
	 * @param int|null $index
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public function loadTemplate( $name, $index = null ) {
		$cls = self::$twig->getTemplateClass( $name, $index );
		
		if ( isset( self::$twig->loadedTemplates[ $cls ] ) ) {
			return self::$twig->loadedTemplates[ $cls ];
		}
		
		if ( ! class_exists( $cls, false ) ) {
			if ( false === ( $cache = $this->get_cached_template( $cls ) ) ) {
				$source = self::$twig->compileSource( self::$twig->getLoader()->getSource( $name ), $name );
				eval( '?>' . $source );
				
				if ( $this->cache ) {
					$this->cache_template( $cls, serialize( $source ) );
				}
			} else {
				eval( '?>' . unserialize( $cache ) );
			}
		}
		
		if ( ! self::$twig->runtimeInitialized ) {
			self::$twig->initRuntime();
		}
		
		return self::$twig->loadedTemplates[ $cls ] = new $cls( $this );
	}
	
	/**
	 * Helper function to fetch cached template
	 *
	 * @param string $name
	 *
	 * @since 0.1
	 * @return bool
	 */
	private function get_cached_template( $name ) {
		if ( ! $this->cache ) {
			return false;
		}
		
		$cache = get_transient( $this->cache_key );
		
		return $cache !== false && isset( $cache[ $name ] ) ? $cache[ $name ] : false;
	}
	
	/**
	 * Helper function to cache template
	 *
	 * @param string $name
	 * @param mixed  $data
	 *
	 * @since 0.1
	 * @return void
	 */
	private function cache_template( $name, $data ) {
		$cache          = get_transient( $this->cache_key );
		$cache[ $name ] = $data;
		set_transient( $this->cache_key, $cache, $this->cache_time );
	}
	
	/**
	 * Forward to local instance::getTokenParsers()
	 * @see   Twig_Environment::getTokenParsers()
	 *
	 * @since 0.1
	 * @return \Twig_TokenParserBrokerInterface
	 */
	public function getTokenParsers() {
		return self::$twig->getTokenParsers();
	}
	
	/**
	 * Forward to local instance::getNodeVisitors()
	 * @see   Twig_Environment::getNodeVisitors()
	 *
	 * @since 0.1
	 * @return \Twig_NodeVisitorInterface[]
	 */
	public function getNodeVisitors() {
		return self::$twig->getNodeVisitors();
	}
	
	/**
	 * Forward to local instance::getUnaryOperators()
	 * @see   Twig_Environment::getUnaryOperators()
	 *
	 * @since 0.1
	 * @return array
	 */
	public function getUnaryOperators() {
		return self::$twig->getUnaryOperators();
	}
	
	/**
	 * Forward to local instance::getBinaryOperators()
	 * @see   Twig_Environment::getBinaryOperators()
	 *
	 * @since 0.1
	 * @return array
	 */
	public function getBinaryOperators() {
		return self::$twig->getBinaryOperators();
	}
	
	/**
	 * Forward to local instance::computeAlternatives()
	 * @see   Twig_Environment::computeAlternatives()
	 *
	 * @param $name
	 * @param $items
	 *
	 * @since 0.1
	 * @return array
	 */
	public function computeAlternatives( $name, $items ) {
		return self::$twig->computeAlternatives( $name, $items );
	}
	
	/**
	 * Forward to local instance::getTags()
	 * @see   Twig_Environment::getTags()
	 *
	 * @since 0.1
	 * @return \Twig_TokenParserInterface[]
	 */
	public function getTags() {
		return self::$twig->getTags();
	}
	
	/**
	 * Forward to local instance::getFunction()
	 * @see   Twig_Environment::getFunction()
	 *
	 * @param string $name
	 *
	 * @since 0.1
	 * @return false|\Twig_Function
	 */
	public function getFunction( $name ) {
		return self::$twig->getFunction( $name );
	}
	
	/**
	 * Forward to local instance::getFunctions()
	 * @see   Twig_Environment::getFunctions()
	 *
	 * @since 0.1
	 * @return \Twig_FunctionInterface[]
	 */
	public function getFunctions() {
		return self::$twig->getFunctions();
	}
	
	/**
	 * Forward to local instance::getBaseTemplateClass()
	 * @see   Twig_Environment::getBaseTemplateClass()
	 *
	 * @since 0.1
	 * @return string
	 */
	public function getBaseTemplateClass() {
		return self::$twig->getBaseTemplateClass();
	}
	
	/**
	 * Forward to local instance::getExtension()
	 * @see   Twig_Environment::getExtension()
	 *
	 * @param string $name
	 *
	 * @since 0.1
	 * @return \Twig_ExtensionInterface
	 */
	public function getExtension( $name ) {
		return self::$twig->getExtension( $name );
	}
	
	/**
	 * Forward to local instance::getExtensions()
	 * @see   Twig_Environment::getExtensions()
	 *
	 * @since 0.1
	 * @return \Twig_ExtensionInterface[]
	 */
	public function getExtensions() {
		return self::$twig->getExtensions();
	}
	
	/**
	 * Forward to local instance::getFilter()
	 * @see   Twig_Environment::getFilter()
	 *
	 * @param string $name
	 *
	 * @since 0.1
	 * @return false|\Twig_Filter
	 */
	public function getFilter( $name ) {
		return self::$twig->getFilter( $name );
	}
	
	/**
	 * Forward to local instance::getFilters()
	 * @see   Twig_Environment::getFilters()
	 *
	 * @since 0.1
	 * @return \Twig_FilterInterface[]
	 */
	public function getFilters() {
		return self::$twig->getFilters();
	}
}