<?php
/**
 * ArticleList widget.
 * Plugin Name: Article List
 * Description: List of articles fetched from Open Content. Can be bound to a "Section" and linked to a "Page".
 * Version: 1.0
 * Author: Infomaker Scandinavia AB
 * Author URI: http://infomaker.se
 */
namespace EwWidgets\Widgets;

use EwTools\Admin\OCWidget;
use EwTools\Models\Section;
use EwTools\Support\Str;
use EwTools\Handler\Templates;
use EwTools\Handler\Pages;
use EwTools\Handler\OpenContent;

/**
 * ArticleList
 *
 * @link    http://infomaker.se
 * @package EwTools\Widgets
 * @since   EwTools\Widgets\ArticleList 0.1
 */
class ArticleList extends OCWidget {
    
    /**
     * Twig-template to use
     *
     * @var string
     */
    protected $template_path = '@tools/widgets/article-list';
    
    /**
     * Contains the default template to use when rendering articles in the list
     *
     * @var string
     */
    private $default_template = 'ocarticle-default.php';
    
    /**
     * Widget fields
     * Contains fields for managing a headline and page to bind it to, a way to fetch data from Open Content
     * and a teaser template to represent every article rendered in the list.
     *
     * @var array
     */
    protected $fields = [
        'headline'       => '',
        'section'        => '',
        'page'           => '',
        'template'       => '',
        'oc_query'       => '',
        'oc_query_start' => 0,
        'oc_query_limit' => 10,
        'oc_query_sort'  => '',
    ];
    
    /**
     * ArticleList constructor.
     *
     * @since 0.1
     */
    public function __construct() {
        $this->setWidgetData( __FILE__ );
        parent::__construct( 'ew-article-list' );
    }
    
    /**
     * Fires when rendering the widget form
     *
     * @param array $view_data
     *
     * @since 0.1
     * @return void
     */
    protected function edit( $view_data ) {
        
        $this->view( 'form', array_replace_recursive( $view_data, [
            'sections'  => Section::toSelect($view_data['section']['value']),
            'pages'     => Pages::getSelectData(),
            'templates' => Templates::getSelectData(),
            'sortings'  => $this->sortingsSelectData()
        ] ) );
    }
    
    /**
     * Fires when rendering widget
     *
     * @param array $view_data
     *
     * @since 0.1
     * @return void
     */
    protected function show( $view_data ) {
        $view_data = array_replace_recursive( $view_data, [
            'articles'      => $this->fetchArticles( $view_data ),
            'section_class' => Section::where('slug', $view_data[ 'section' ] )->first()->get('class_name')
        ] );
        
        $this->view( 'widget', $view_data );
    }
    
    /**
     * Function to fetch the articles from Open Content
     *
     * @param $view_data
     *
     * @since 0.1
     * @return array
     */
    private function fetchArticles( $view_data ) {
        
        // TODO: Implement fetching of articles!!!
        return [];
        $search_result = OpenContent::search( $this->convertWidgetQuery( $view_data ) );
        $oc_articles   = [];
        
        if( is_array( $search_result ) ) {
            unset( $search_result[ 'facet' ] );
            unset( $search_result[ 'hits' ] );
            unset( $search_result[ 'duration' ] );
            $oc_articles = $search_result;
        }
        
        if( ! Str::isEmpty( $view_data[ 'template' ] ) ) {
            return $this->generateFromTemplate( $oc_articles, Templates::getPath( $view_data[ 'template' ] ) );
        }
        
        return [];
    }
    
    /**
     * Run the article through the template to generate HTML
     *
     * @param OC_Article $oc_articles
     * @param string     $template
     *
     * @since 0.1
     * @return array
     */
    private function generateFromTemplate( $oc_articles, $template ) {
        $articles = [];
        ob_start();
        foreach ( $oc_articles as $index => $article ) {
            include $template;
            $articles[] = ob_get_contents();
            ob_clean();
        }
        ob_end_clean();
        
        return $articles;
    }
    
    /**
     * Convert the form-data into values readable by OpenContent::search()
     *
     * @param $view_data
     *
     * @since 0.1
     * @return array
     */
    protected function convertWidgetQuery( $view_data ) {
        return [
            'query'   => $view_data[ 'oc_query' ],
            'start'   => $view_data[ 'oc_query_start' ],
            'limit'   => $view_data[ 'oc_query_limit' ],
            'sorting' => $view_data[ 'oc_query_sort' ],
            'use_raw' => true
        ];
    }
}

ArticleList::register();